__all__ = (
    "TEST_CYCLES",
    "URLS",
    "URLS_COUNT",
)

URLS_COUNT = 10000

URLS = [
    "http://www.google.co.uk",
    "http://www.v2.google.co.uk",
    "http://www.google.co.uk:8001/lorem-ipsum/",
    "http://www.me.cloudfront.net",
    "http://www.v2.forum.tech.google.co.uk:8001/lorem-ipsum/",
    "https://pantheon.io/",
    "delusionalinsanity.com",
    "www.baidu.com.cn",
    "i.dont.exist",
    "http://delusionalinsanity.com",
]

TEST_CYCLES = 10000
